-- Adminer 3.1.0-dev MySQL dump

SET NAMES utf8;
SET foreign_key_checks = 0;
SET time_zone = 'SYSTEM';
SET sql_mode = 'NO_AUTO_VALUE_ON_ZERO';

USE `doctrine`;

DROP TABLE IF EXISTS `article`;
CREATE TABLE `article` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `category_id` int(11) DEFAULT NULL,
  `title` varchar(100) COLLATE utf8_unicode_ci NOT NULL,
  `text` longtext COLLATE utf8_unicode_ci NOT NULL,
  `published` date NOT NULL,
  `visits` int(11) NOT NULL,
  PRIMARY KEY (`id`),
  KEY `Article_category_id_published_idx` (`category_id`,`published`),
  CONSTRAINT `article_ibfk_1` FOREIGN KEY (`category_id`) REFERENCES `category` (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=564 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

INSERT INTO `article` (`id`, `category_id`, `title`, `text`, `published`, `visits`) VALUES
(174,	1,	'IT Underground',	'<p>I have conducted a talk <em>Web Security with PHP</em> on <a href=\"http://www.itunderground.org/cz_en2/\">IT Underground Prague</a>. Here are the <a href=\"http://php.vrana.cz/data/security.zip\">slides, examples and tasks</a>. Correct answers for tasks are hidden in HTML comments. <ins datetime=\"2006-04-04T17:00+02:00\">Presentation is available also in <a href=\"http://talks.php.net/show/vrana-security\">PHP Presentation format</a>.</ins></p>\r\n',	'2006-02-24',	177),
(446,	1,	'Storing multilingual records in the MySQL database',	'<p><em>This article was published in the <a href=\"http://www.phparch.com/magazine/index/95\">April 2009 issue of php|architect</a>.</em></p>\n<p>Multilingual applications use two types of texts requiring translation. The first ones are messages without a relation to the data (for example the title \"Contents\") which can easily be translated by <a href=\"http://www.gnu.org/software/gettext/\">GNU gettext</a> or by a simple database table. The second type  are texts related to the data (for example a German and Italian title for the same product with a given price). This article will examine database strategies for this second type of text.</p>\n<p>Multilingual applications should use simple SQL queries (to avoid bugs), keep the performance high (which mainly means to utilize indexes) and produce the exact output (sort according to each language collation and respect the fulltext threshold). In additionally database consistency should be maintained, for example to preserve uniqueness of URLs. This article compares three different approaches for storing multilingual records considering these goals. In addition, the complexity of adding a new language and working with untranslated texts are covered.</p>\n<p>This article demonstrates all approaches on a simple table of products with translatable names, URLs and descriptions, group identifiers and prices (common for all languages). The application needs to filter the products by the group identifier and then sort them by the name and we will perform a fulltext search on the name and description in a specified language. Therefore, in a single language environment, the table would look like this:</p>\n<img src=\"data/multilingual0.png\" width=\"84\" height=\"128\" alt=\"database schema\" class=\"right\" />\n<pre class=\"jush-sql\">CREATE TABLE `product` (\n	`product_id` int(11) NOT NULL auto_increment,\n	`group_id` int(11) NOT NULL,\n	`name` varchar(50) NOT NULL,\n	`url` varchar(50) NOT NULL,\n	`price` decimal(9, 2) NOT NULL,\n	`description` text NOT NULL,\n	UNIQUE KEY `url` (`url`),\n	KEY `group_id` (`group_id`, `name`),\n	FULLTEXT KEY `name` (`name`, `description`),\n	PRIMARY KEY (`product_id`)\n);\n</pre>\n<p>The unique index above URL guarantees that there can be only one product at each location. The query to retrieve the products in a given group sorted by the name is simple (PHP syntax is used):</p>\n<pre class=\"jush-php_sql\">\"\n	SELECT `url`, `name`, `price`\n	FROM `product`\n	WHERE `group_id` = $group_id\n	ORDER BY `name`\n	LIMIT 30\n\"</pre>\n<p>The usage of indexes is of course correct:</p>\n<table cellspacing=\"0\" cellpadding=\"3\">\n<thead><tr><th>id</th><th>select_type</th><th>table</th><th>type</th><th>possible_keys</th><th>key</th><th>key_len</th><th>ref</th><th>rows</th><th>Extra</th></tr></thead>\n<tr><td>1</td><td>SIMPLE</td><td>product</td><td>ref</td><td>group_id</td><td>group_id</td><td>4</td><td>const</td><td>9</td><td>Using where</td></tr>\n</table>\n\n<h3><a name=\"separate\"></a>Translations in a separate table</h3>\n<p>We can store the translatable columns in a separate table:</p>\n<img src=\"data/multilingual1.png\" width=\"293\" height=\"186\" alt=\"database schema\" class=\"right\" />\n<pre class=\"jush-sql\">CREATE TABLE `language` (\n	`language_id` char(2) NOT NULL,\n	`collation` varchar(64) NOT NULL,\n	PRIMARY KEY (`language_id`)\n);\n\nCREATE TABLE `product` (\n	`product_id` int(11) NOT NULL auto_increment,\n	`group_id` int(11) NOT NULL,\n	`price` decimal(9, 2) NOT NULL,\n	KEY `group_id` (`group_id`),\n	PRIMARY KEY (`product_id`)\n);\n\nCREATE TABLE `product_translation` (\n	`product_id` int(11) NOT NULL,\n	`language_id` char(2) NOT NULL,\n	`name` varchar(50) NOT NULL,\n	`url` varchar(50) NOT NULL,\n	`description` tinytext NOT NULL,\n	UNIQUE KEY `language_id_2` (`language_id`, `url`),\n	KEY `language_id` (`language_id`, `name`),\n	FULLTEXT KEY `name` (`name`, `description`),\n	FOREIGN KEY (`language_id`) REFERENCES `language` (`language_id`),\n	FOREIGN KEY (`product_id`) REFERENCES `product` (`product_id`),\n	PRIMARY KEY (`product_id`, `language_id`)\n);\n</pre>\n<p>(The <code>`language`.`collation`</code> column contains one of the collations returned by the <kbd>SHOW COLLATION</kbd> command and we need it to properly sort the results.)</p>\n<p>With this schema, a simple query to retrieve the products in a given group sorted by the name is not so simple:</p>\n<pre class=\"jush-php_sql\">\"\n	SELECT `product_translation`.`url`, `product_translation`.`name`, `product`.`price`\n	FROM `product`\n	INNER JOIN `product_translation` USING (`product_id`)\n	WHERE `product`.`group_id` = $group_id AND `product_translation`.`language_id` = \'$language_id\'\n	ORDER BY `product_translation`.`name` COLLATE $collation\n	LIMIT 30\n\"</pre>\n<p>(Variable <code>$collation</code> is taken from <code>`language`.`collation`</code> in the application initialization.)</p>\n<p>The query is complex and its execution plan is bad:</p>\n<table cellspacing=\"0\" cellpadding=\"3\">\n<thead><tr><th>id</th><th>select_type</th><th>table</th><th>type</th><th>possible_keys</th><th>key</th><th>key_len</th><th>ref</th><th>rows</th><th>Extra</th></tr></thead>\n<tr><td>1</td><td>SIMPLE</td><td>product</td><td>ref</td><td>PRIMARY, group_id</td><td>group_id</td><td>4</td><td>const</td><td>12</td><td>Using temporary; Using filesort</td></tr>\n<tr><td>1</td><td>SIMPLE</td><td>product_translation</td><td>eq_ref</td><td>PRIMARY, language_id_2, language_id</td><td>PRIMARY</td><td>10</td><td>product.product_id, const</td><td>1</td><td>Using where</td></tr>\n</table>\n<p>Because of filesort, MySQL has to read all entries and cannot use the LIMIT clause to skip unnecessary rows. It is <code>N * log N</code> where as to perform the ordering with LIMIT clause would be a constant! MySQL cannost use the index on <code>(language_id, name)</code> for sorting at all because we need a different collation for every language.</p>\n<p>What about <strong>fulltext search</strong>? MySQL natural language search ignores words occurring in at least half of the rows. Because all languages are mixed in one column, this feature will likely exclude most words and thus be useless. Fulltext search can be good in some scenarios but it has negative impact on performance. Moreover, MySQL cannot use a fulltext index together with a normal index so the fulltext must search in all languages and then restrict the results by hand.</p>\n<p>In a real-world application, we have to deal with <strong>untranslated texts</strong>. Usually, we use some default language in the place of them. With this table schema, we have two options available to solve this problem:</p>\n<ol>\n<li>One option is to copy the default language version of all texts to the newly added language. This greatly complicates adding a new language to the application. It also complicates adding a new product. <strong>Unique index</strong> above URL would work but then we would have to insert all translations in a transaction and rollback the transaction in a case of conflict.</li>\n<li>The second option is to use the default language version if a translation does not exist. This makes <strong>Adding a language</strong> very simple (just inserting a row in the language table) but the  so called \"simple query\" gets complicated even more\n<pre class=\"jush-php_sql\">\"\n	SELECT IFNULL(`product_translation`.`url`, `product_default`.`url`) AS `url`, IFNULL(`product_translation`.`name`, `product_default`.`name`) AS `name`, `product`.`price`\n	FROM `product`\n	INNER JOIN `product_translation` `product_default` USING (`product_id`)\n	LEFT JOIN `product_translation` ON `product`.`product_id` = `product_translation`.`product_id` AND `product_translation`.`language_id` = \'$language_id\'\n	WHERE `product`.`group_id` = $group_id AND `product_default`.`language_id` = \'$default_id\'\n	ORDER BY IFNULL(`product_translation`.`name`, `product_default`.`name`) COLLATE $collation\n	LIMIT 30\n\"</pre>\nUnique index above URL is useless as we would have to check for existence of the same URL in a trigger or a stored procedure at the time of inserting, updating and deleting a product translation.\n</li>\n</ol>\n\n<h3><a name=\"copy\"></a>Data copy</h3>\n<p>This approach copies also the data common to all languages.</p>\n<img src=\"data/multilingual2.png\" width=\"285\" height=\"218\" alt=\"database schema\" class=\"right\" />\n<pre class=\"jush-sql\">CREATE TABLE `language` (\n	`language_id` char(2) NOT NULL,\n	`collation` varchar(64) NOT NULL,\n	PRIMARY KEY (`language_id`)\n);\n\nCREATE TABLE `product` (\n	`product_id` int(11) NOT NULL auto_increment,\n	PRIMARY KEY (`product_id`)\n);\n\nCREATE TABLE `product_translation` (\n	`product_id` int(11) NOT NULL,\n	`language_id` char(2) NOT NULL,\n	`group_id` int(11) NOT NULL,\n	`name` varchar(50) NOT NULL,\n	`url` varchar(50) NOT NULL,\n	`price` decimal(9, 2) NOT NULL,\n	`description` text NOT NULL,\n	UNIQUE KEY `language_id_2` (`language_id`, `url`),\n	KEY `language_id` (`language_id`, `group_id`, `name`),\n	FULLTEXT KEY `name` (`name`, `description`),\n	FOREIGN KEY (`product_id`) REFERENCES `product` (`product_id`),\n	FOREIGN KEY (`language_id`) REFERENCES `language` (`language_id`),\n	PRIMARY KEY (`product_id`, `language_id`)\n);\n</pre>\n<p>(The product table can be just a sequence.)</p>\n<p>The group identifier and the price are copied to all languages. Technically a trigger can achieve it, however we have to define a trigger on all translatable tables in the database and it has to respect all columns in the table (so addition or modification of a table would be difficult). The data also takes more space so the working set is less likely to fit in memory.</p>\n<p>The <strong>simple query</strong> remains simple:</p>\n<pre class=\"jush-php_sql\">\"\n	SELECT `url`, `name`, `price`\n	FROM `product_translation`\n	WHERE `language_id` = \'$language_id\' AND `group_id` = $group_id\n	ORDER BY `name` COLLATE $collation\n	LIMIT 30\n\"</pre>\n<p>Even the execution plan would be good only if it were not for the COLLATE clause:</p>\n<table cellspacing=\"0\" cellpadding=\"3\">\n<thead><tr><th>id</th><th>select_type</th><th>table</th><th>type</th><th>possible_keys</th><th>key</th><th>key_len</th><th>ref</th><th>rows</th><th>Extra</th></tr></thead>\n<tr><td>1</td><td>SIMPLE</td><td>product_translation</td><td>ref</td><td>language_id_2, language_id</td><td>language_id</td><td>10</td><td>const, const</td><td>9</td><td>Using where; Using filesort</td></tr>\n</table>\n<p>The COLLATE is causing a filesort which is bad for the LIMIT clause. The query would be fast without it but the results would not respect the sorting order specific for the language.</p>\n<p>Both problems with <strong>fulltext search</strong> remain.</p>\n<p>We have to solve the same dilemma as in the previous approach regarding <strong>untranslated texts</strong> and <strong>adding a new language</strong>. Either the queries will be complicated and slower or the language addition will be painful. <strong>Unique index</strong> would not work in the case of falling back to the default language or we would have to do all modifications in a transaction when copying the default language.</p>\n\n<h3><a name=\"inside\"></a>Translation directly in the data table</h3>\n<p>The previous approaches have one basic problem - they are storing different data (texts with miscellaneous collations) to the same column. We have to separate the languages into the individual columns to solve this basic problem:</p>\n<img src=\"data/multilingual3.png\" width=\"224\" height=\"182\" alt=\"database schema\" class=\"right\" />\n<pre class=\"jush-sql\">CREATE TABLE `language` (\n	`language_id` char(2) NOT NULL,\n	`collation` varchar(64) NOT NULL,\n	PRIMARY KEY (`language_id`)\n);\n\nCREATE TABLE `product` (\n	`product_id` int(11) NOT NULL auto_increment,\n	`group_id` int(11) NOT NULL,\n	`name_en` varchar(50) NOT NULL,\n	`name_cs` varchar(50) collate utf8_czech_ci NOT NULL,\n	`url_en` varchar(50) NOT NULL,\n	`url_cs` varchar(50) collate utf8_czech_ci NOT NULL,\n	`price` decimal(9, 2) NOT NULL,\n	`description_en` text NOT NULL,\n	`description_cs` text collate utf8_czech_ci NOT NULL,\n	UNIQUE KEY `url_en` (`url_en`),\n	UNIQUE KEY `url_cs` (`url_cs`),\n	KEY `group_id_en` (`group_id`, `name_en`),\n	KEY `group_id_cs` (`group_id`, `name_cs`),\n	FULLTEXT KEY `name_cs` (`name_cs`, `description_cs`),\n	FULLTEXT KEY `name_en` (`name_en`, `description_en`),\n	PRIMARY KEY (`product_id`)\n);\n</pre>\n<p>(Notice the different collations for columns with translations.)</p>\n<p>Each translatable field gets its own column in each language. The same holds for indexes - there are more of them but they are smaller.</p>\n<p>The table is not nice but the <strong>simple query</strong> remains simple and produces correct results:</p>\n<pre class=\"jush-php_sql\">\"\n	SELECT `url_$language_id`, `name_$language_id`, `price`\n	FROM `product`\n	WHERE `group_id` = $group_id\n	ORDER BY `name_$language_id`\n	LIMIT 30\n\"</pre>\n<p>(The <code>$language_id</code> variable is whitelisted from <code>`language`.`language_id`</code>.)</p>\n<p>It is also as fast as a single-language query:</p>\n<table cellspacing=\"0\" cellpadding=\"3\">\n<thead><tr><th>id</th><th>select_type</th><th>table</th><th>type</th><th>possible_keys</th><th>key</th><th>key_len</th><th>ref</th><th>rows</th><th>Extra</th></tr></thead>\n<tr><td>1</td><td>SIMPLE</td><td>product</td><td>ref</td><td>group_id_en, group_id_cs</td><td>group_id_en</td><td>4</td><td>const</td><td>11</td><td>Using where</td></tr>\n</table>\n<p>Both problems with <strong>fulltext search</strong> disappear. Each index has separate statistics and there is no need to filter by language identifier when performing the search.</p>\n<p>Adding a language definitely will be a difficult operation so we do not need to solve the dilemma whether to store the default language to <strong>untranslated texts</strong>. It can be stored to keep query simplicity and speed. <strong>Unique indexes</strong> above URLs simply work.</p>\n<p><strong>Adding a new language</strong> will be a complex operation when this approach is used. We have to go through all tables and add a column for each language field. We also have to create new indexes. We can achieve it by a stored procedure or by a PHP function:</p>\n<?php\n/** Addition of language columns to all tables\n* @param string language identifier\n* @param string default language identifier\n* @param string way of ordering texts\n* @return null execution of commands ALTER TABLE and UPDATE\n* @copyright Jakub Vrana, http://php.vrana.cz\n*/\nfunction add_language($language_id, $default_id = \"en\", $collation = \"\") {\n	$result = mysql_query(\"SHOW TABLES\");\n	while ($row = mysql_fetch_row($result)) {\n		\n		// addition and copy of columns\n		$alter = array();\n		$update = array();\n		$result1 = mysql_query(\"SHOW COLUMNS FROM $row[0]\");\n		while ($row1 = mysql_fetch_assoc($result1)) {\n			if (substr($row1[\"Field\"], -3) == \"_$default_id\") {\n				$field = substr($row1[\"Field\"], 0, -2) . $language_id;\n				$null = ($row1[\"Null\"] == \"NO\" ? \" NOT NULL\" : \"\");\n				$alter[] = \"ADD $field $row1[Type]\" . ($collation ? \" COLLATE $collation\" : \"\") . \"$null AFTER $row1[Field]\";\n				$update[] = \"$field = $row1[Field]\";\n			}\n		}\n		mysql_free_result($result1);\n		if ($alter) {\n			mysql_query(\"ALTER TABLE $row[0] \" . implode(\", \", $alter));\n			mysql_query(\"UPDATE $row[0] SET \" . implode(\", \", $update));\n			\n			// addition of indexes\n			$indexes = array();\n			$result1 = mysql_query(\"SHOW INDEXES FROM $row[0]\");\n			while ($row1 = mysql_fetch_assoc($result1)) {\n				$type = ($row1[\"Index_type\"] == \"FULLTEXT\" ? \"FULLTEXT\" : ($row1[\"Non_unique\"] ? \"INDEX\" : \"UNIQUE\"));\n				$indexes[$type][$row1[\"Key_name\"]] .= \"$row1[Column_name], \";\n			}\n			mysql_free_result($result1);\n			foreach ($indexes as $type => $type_indexes) {\n				foreach ($type_indexes as $index) {\n					if (strpos($index, \"_$default_id, \")) {\n						mysql_query(\"ALTER TABLE $row[0] ADD $type (\" . substr(str_replace(\"_$default_id, \", \"_$language_id, \", $index), 0, -2) . \")\");\n					}\n				}\n			}\n		}\n	}\n	mysql_free_result($result);\n}\n?>\n<p>The maximum row size in MySQL is about 64 KB so with big tables and many languages, this limit could be reached. In this case, it is possible to separate some columns to a different table with one-to-one relationship. Another approach would be to store each language in a separate table and copy common columns the same way as we did in the Data copy approach. This however involves problems with trigger maintenance.</p>\n\n<div class=\"right\" style=\"width: 25em; background-color: #DDDDFF; border: 1px solid Navy; padding: 0 1em;\">\n<p><strong>Other databases</strong></p>\n<p>Other database systems such as Oracle or PostgreSQL do not have support for a collation definition in each column and use a different approach for a language-dependent sorting. They define (PostgreSQL through an extension) an <kbd>nlssort</kbd> function accepting a string to sort and a language identifier to sort by. The database systems can use this function together with partial function indexes to sort the products effectively in the Data copy schema. However, we still have to define the indexes at the time of adding a language.</p>\n</div>\n\n<h3>Final thoughts</h3>\n<ul>\n<li>For messages with no relation to the data (mentioned in the beginning of this article) we can use the same database schema to store translations.  This way we can use the same infrastructure for translating both kinds of texts.</li>\n<li>Translators need a list of texts required for translation. We cannot generate this list by comparing messages that are the same as the default language because some translations are simply the same as original. We can maintain this list in a separate table.</li>\n<li>In the ideal world, all the translations would be stored in a single cell with complex data type. This data type would know how to sort the data in each language and how to use them in index. It would also know how to handle untranslated texts. However, we have to split the cell to either rows or columns in the world of relational databases.</li>\n</ul>\n\n<h3>Conclusion</h3>\n<p>Let\'s name pros and cons of all three approaches:</p>\n<table cellspacing=\"0\" cellpadding=\"3\">\n<thead><tr><th>Translations in a separate table</th><th>Data copy</th><th>Translation in the data table</th></tr></thead>\n<tr valign=\"top\">\n<td>- slow sorting<br />- slow fulltext search<br />- complex SQL queries<br /><br />Copy of default language:<br />- complex language addition<br />- complex product insertion<br />- extra table for untranslated texts<br /><br />Fallback to default language:<br />+ simple language addition<br />- even more complex SQL queries<br />- complex URL modifications</td>\n<td>- slow or inaccurate sorting<br />- slow fulltext search<br />- bigger footprint<br />- difficult table altering<br /><br />Copy of default language:<br />+ simple queries<br />- complex language addition<br />- complex product insertion<br />- extra table for untranslated texts<br /><br />Fallback to default language:<br />+ simple language addition<br />- complex SQL queries<br />- complex any URL modification</td>\n<td>- variable database structure<br />- very complex language addition<br /><br />Copy of default language:<br />+ simple queries<br />+ fast sorting<br />+ fast fulltext search<br />- extra table for untranslated texts</td>\n</tr>\n</table>\n<p>The third approach has a big handicap in the variable database structure (the user adding a language must have the ALTER privilege). It exceeds the other approaches in all other aspects and at the end of a day, we have to do the least work and still get a solution with the highest performance.</p>\n\n<h3><a name=\"author\"></a>About the author</h3>\n<p>Jakub Vrána is one of the authors of official PHP manual, and he uses MySQL for developing web applications in PHP. He also teaches the MySQL basics at Charles University and conducts commercial trainings. He is the founder of compact MySQL management tool phpMinAdmin. Jakub lives in Prague, Czech Republic.</p>\n',	'2009-08-24',	0),
(499,	1,	'Architecture of Adminer',	'<p><em>This article was published in the <a href=\"http://phpa-bender.phparch.com/magazine/index/104\">August 2009 issue of php|architect</a>.</em></p>\n<p><a href=\"http://www.adminer.org/\"><strong>Adminer</strong></a> is a feature complete MySQL administration tool with the aim of compactness. The whole application consists of a single file as small as possible. This request influences the code design of Adminer and this article describes the way of reaching that target.</p>\n<p>As a freelance PHP programmer, I often get a task to check some problem or implement new functionality at a strange site. I usually get only FTP credentials without further documentation about the application or about the client\'s hosting. If the task consists of some database manipulation then I have to discover or modify the database structure without any tools or remote access to the database server. I can usually install <strong>phpMyAdmin</strong> in the web space but it currently consists of 11 MB in 666 files and I became quickly tired of uploading this package with every new task (and removing it after the task completion).</p>\n<p>Therefore, I started to create a compact alternative of phpMyAdmin consisting of a single file easy to upload and delete anywhere. Initially, it was a simple tool with basic commands such as the table structure, select, SQL command and export. Later, I have added more and more features so now it is a feature complete MySQL administration tool including even the MySQL 5.1 features (events and table partitioning). Adminer implements some features (such as triggers or stored routines) in a more comfortable way than the bigger competitor but the basic goal of a single file is still valid.</p>\n<p><b>Note:</b> The project\'s original name was <strong>phpMinAdmin</strong>. I chose that name as a joke to emphasize the fact that phpMinAdmin is a minimalistic version of the famous PHP MySQL tool. (By the way do you use <strong>Lynx</strong> or <strong>Links</strong>?) This name was however confusing and invoked the feeling that phpMinAdmin is only a weaker alternative of phpMyAdmin.</p>\n<p style=\"text-align: center;\"><img src=\"data/adminer.png\" width=\"633\" height=\"530\" alt=\"Adminer Table Structure\" /></p>\n\n<h3>Compilation</h3>\n<p>When the size of <code>adminer.php</code> started to grow then I have realized that it would be wise to split the file for manageable development. Nevertheless, I still wanted the all-in-one file so I have created a compilation script that joins the files back together.</p>\n<p>Because everything lies in one file, Adminer cannot use the usual URL <code>alter.php?table=X</code> and the action must be passed in the query string. Instead of common <code>?action=alter&amp;table=X</code>, I have used the more compact alternative <code>?alter=X</code>. The central script is then a sequence of <code class=\"jush-php\">if (isset($_GET[\"...\"]))</code> instead of one big <code class=\"jush-php\">switch ($_GET[\"action\"])</code>. There is an <a href=\"http://www.php.net/manual/en/function.include.php\"><kbd>include</kbd></a> of the specific functionality inside the <code>if</code> and these includes are compiled to a one file in the build process:</p>\n<?php\nfunction include_file($match) {\n	$file = file_get_contents($match[1]);\n	$token = end(token_get_all($file));\n	$php = (is_array($token) && in_array($token[0], array(T_CLOSE_TAG, T_INLINE_HTML)));\n	return \"?>\\n$file\" . ($php ? \"<?php\" : \"\");\n}\n$file = preg_replace_callback(\'~include \"([^\"]+)\";~\', \'include_file\', $file);\n?>\n<p>This code is not universally usable because it finds only the <code class=\"jush-php\">include \"\"</code> variant. It would be necessary to process the file with <a href=\"http://www.php.net/manual/en/function.token-get-all.php\"><kbd>token_get_all</kbd></a> function and find all <a href=\"http://www.php.net/manual/en/function.include.php\"><kbd>include</kbd></a> variants. The <code>_once</code> variants would be more difficult. There can be also a global level <a href=\"http://www.php.net/manual/en/function.return.php\"><kbd>return</kbd></a>.</p>\n<p>Adminer uses also some external files – style sheet, shortcut icon and couple of images. There are several possible approaches of accessing these files:</p>\n<ol>\n<li>They can be integrated in the main HTML code – style sheet by <code class=\"jush\">&lt;style&gt;</code> tag, images by utilizing the <code>data:</code> protocol. The problem with this approach is that Internet Explorer &lt; 8 does not support this protocol. The second problem is that the browser would need to transfer these data with each page repeatedly.</li>\n<li>The files can be downloaded from an external server. It would be problematic if the server is unreachable or if the computer running Adminer is without the Internet connection. This approach is however used with syntax highlighting of SQL queries, which is an optional feature – Adminer uses the JavaScript Syntax Highlighter <a href=\"http://jush.sourceforge.net/\"><strong>JUSH</strong></a> for this task.</li>\n<li>The files can be integrated in the source code and a special parameter would serve them. Did you ever notice the PHP logo in the output of <a href=\"http://www.php.net/manual/en/function.phpinfo.php\"><kbd>phpinfo</kbd></a>? PHP downloads it from the local server with a special query string <code>?=PHP…</code>. Adminer uses a similar approach and saves the files Base64 encoded in the source code (the encoding is not necessary but it simplifies the script editing in common text editors). This approach allows the HTTP caching of the external files which Adminer utilizes.</li>\n</ol>\n<p><strong>Note:</strong> PHP 5.3 comes with a support for PHP archives through the <em>phar</em> extension. With this extension, it is possible to pack several files to one archive and access the files inside the archive from PHP. Usage of the <a href=\"http://www.php.net/manual/en/phar.webphar.php\"><kbd>webPhar</kbd></a> method allows packing most of the PHP applications to one archive and there are instructions for creating <em>phar</em> version of phpMyAdmin in the <a href=\"http://www.php.net/manual/en/phar.using.intro.php\">PHP manual</a>. Adminer does not use this extension and works with PHP &gt;= 4.3 and PHP &gt;= 5.0.</p>\n\n<h3>Minification</h3>\n<p>The compilation script also minifies the code – it removes comments and trims white space. The minification code is based on a work of David Grudl who uses it for his brilliant libraries <a href=\"http://texy.info/\"><strong>Texy</strong></a>, <a href=\"http://dibiphp.com\"><strong>dibi</strong></a> and <a href=\"http://nettephp.com\"><strong>Nette</strong></a>. This function is more effective than the internal function <a href=\"http://www.php.net/manual/en/function.php-strip-whitespace.php\"><kbd>php_strip_whitespace</kbd></a>.</p>\n<?php\nfunction php_shrink($input) {\n	$set = array_flip(preg_split(\'//\', \'!\"#$&\\\'()*+,-./:;<=>?@[\\]^`{|}\'));\n	$space = \'\';\n	$output = \'\';\n	foreach (token_get_all($input) as $token) {\n		if (!is_array($token)) {\n			$token = array(0, $token);\n		}\n		if ($token[0] == T_COMMENT || $token[0] == T_WHITESPACE) {\n			$space = \"\\n\";\n		} else {\n			if (isset($set[substr($output, -1)]) || isset($set[$token[1][0]])) {\n				$space = \'\';\n			}\n			$output .= $space . $token[1];\n			$space = \'\';\n		}\n	}\n	return $output;\n}\n?>\n<p>Current version also shortens the names of user variables and functions. It finds the variables and functions in the source code with <a href=\"http://www.php.net/manual/en/function.token-get-all.php\"><kbd>token_get_all</kbd></a> function and replaces them by shorter identifiers. The shortening process skips the internal PHP variables and functions. It would be possible to shorten them too but it would be more complicated and it would have a performance penalty. Some PHP variables have their super global feature so it would be necessary to globalize their aliases inside the functions.</p>\n<p>The internal functions can be enclosed in envelope functions with shorter names. This approach is however problematic because the PHP functions can have optional parameters without the default values (example of this function is <a href=\"http://www.php.net/manual/en/function.fwrite.php\"><kbd>fwrite</kbd></a> which detects the number of passed arguments instead of their values). Universal function <a href=\"http://www.php.net/manual/en/function.func-get-args.php\"><kbd>func_get_args</kbd></a> does not work with the references on the contrary.</p>\n<p>Alternative approach would be to define a variable for an internal function (e.g. <code class=\"jush-php\">$fw = \'fwrite\'</code>) and call the function through this variable: <code class=\"jush-php\">$fw($fp, $string)</code>. Adminer however does not use neither approach because it would slow the application down.</p>\n<p>The minification trims white space also from CSS and <a href=\"http://code.google.com/p/jsmin-php/\">PHP version of <strong>JSMin</strong></a> minifies the JavaScript code too.</p>\n<p>The file would be much smaller if it would be compressed. PHP supports several compression algorithms but only through an extension. I was thinking about a simple decompression function written in PHP which will unpack the rest of the file but it will need to <a href=\"http://www.php.net/manual/en/function.eval.php\"><kbd>eval</kbd></a> the decompressed code and everybody knows that <kbd>eval</kbd> is evil (one reason is that it is not compatible with PHP accelerators). Therefore, the code is not compressed but only minified.</p>\n\n<h3>Translations</h3>\n<p>I develop Adminer in English. There is also a Czech version (which is my native language) from the beginning and Adminer is currently available in 11 languages. All messages are served by a simple function, which checks the current language and returns the correspondent translation.</p>\n<p>A simple function detects the language from the <code class=\"jush-http\">Accept-Language</code> header:</p>\n<?php\nfunction acceptable_language($translations) {\n	$accept_language = array();\n	preg_match_all(\'~([-a-z]+)(;q=([0-9.]+))?~\', strtolower($_SERVER[\"HTTP_ACCEPT_LANGUAGE\"]), $matches, PREG_SET_ORDER);\n	foreach ($matches as $match) {\n		$accept_language[$match[1]] = (isset($match[3]) ? $match[3] : 1);\n	}\n	arsort($accept_language);\n	foreach ($accept_language as $lang => $q) {\n		if (isset($translations[$lang])) {\n			return $lang;\n		}\n		$lang = preg_replace(\'~-.*~\', \'\', $lang);\n		if (!isset($accept_language[$lang]) && isset($translations[$lang])) {\n			return $lang;\n		}\n	}\n	return false;\n}\n?>\n<p>The caller of this function saves the language to a cookie after the first detection. A user can change the language anytime while staying on the same page. Saving the page language to a cookie is a very bad practice on a public site because the search engines then can index only one version of the page. It is however acceptable in Adminer because the search engines do not index the pages and the main content of the page (e.g. the table structure) is the same in all languages. The reward for storing the language in a cookie is a simpler URL while it is still possible to enforce the language by passing the language identifier in the query string.</p>\n<p>Translations are stored in a simple array where the key is a message identifier (which is an English version of the message and it is used if the translation does not exist) and the value is a string with the translation. The value is more interesting with messages containing a number (e.g. “1 row” or “2 rows”). Most languages have different singular and plural form but some languages (e.g. Czech or Russian) use more plural forms depending on the number (e.g. Czech “1 pivo”, “2 piva” or “5 piv” for beers). The messages with numbers are stored in an array instead of string and Adminer contains a simple logic for each language choosing the right form depending on the number.</p>\n<p>By default, Adminer comes with all languages. The compilation converts the translation array to values only and changes the identifiers to numbers to save some bytes so each translation takes about 4 KB. The compilation script is able to create also a single language version, which removes the translating function and the language detection completely. This file takes only 111 KB for the English version of Adminer 1.10.1.</p>\n\n<h3>Database extension abstraction</h3>\n<p>PHP allows connecting to a MySQL database through three extensions: <em>mysql</em>, <em>mysqli</em> and <em>pdo_mysql</em>. Adminer supports all of them and contains a simple abstraction layer for these extensions. The abstraction layer emulates the subset of <em>mysqli</em> and <em>mysqli_result</em> classes for all three extensions.</p>\n<p>PDO support is little tricky because PDO uses exceptions for reporting connection errors. Adminer cannot use the usual <code>try</code> block to catch these exceptions because it runs also on PHP 4. Everything is in a single file so the PDO support cannot be conditionally included only in PHP 5. The solution is to use <a href=\"http://www.php.net/manual/en/function.set-exception-handler.php\"><kbd>set_exception_handler</kbd></a> function to process the connection errors. PHP 4 also does not support class constants on the syntax level so the source code uses the numerical values of the constants instead.</p>\n\n<h3>Security</h3>\n<p>The most important part of the security in Adminer is surprisingly the defense against Cross-site Request Forgery. CSRF allows an attacker to perform actions in the name of a logged-in user. Adminer stores the login information in a standard session so PHP can send the session identifier in a cookie and the defense against CSRF is necessary. Adminer sends a token with each form and checks this token before performing the operation of the form. A token protects even the logout action which is often forgotten in other web applications.</p>\n<p>Very important is also the protection against Cross-site Scripting. Outputting the HTML code stored in the database could reveal sensitive information to the attacker so <a href=\"http://www.php.net/manual/en/function.htmlspecialchars.php\"><kbd>htmlspecialchars</kbd></a> function escapes every output.</p>\n<p><span class=\"callout\">Adminer has also a protection against the SQL injection but it is not in the name of security</span> – if the attacker already logs-in to the Adminer then she could execute any SQL code by the SQL command page. Therefore, the protection against the SQL injection is only a side effect of proper manipulation with the user input – a user can pass any data to the database. If Adminer detects the <a href=\"http://www.php.net/manual/en/info.configuration.php#ini.magic-quotes-gpc\"><kbd>magic_quotes_gpc</kbd></a> then it neutralizes this directive and uses a proper escaping function only when passing the data to MySQL.</p>\n<?php\nif (get_magic_quotes_gpc()) {\n	$process = array(&$_GET, &$_POST, &$_COOKIE, &$_REQUEST);\n	while (list($key, $val) = each($process)) {\n		foreach ($val as $k => $v) {\n			unset($process[$key][$k]);\n			if (is_array($v)) {\n				$process[$key][stripslashes($k)] = $v;\n				$process[] = &$process[$key][stripslashes($k)];\n			} else {\n				$process[$key][stripslashes($k)] = stripslashes($v);\n			}\n		}\n	}\n	unset($process);\n}\n?>\n<p>Calling of <a href=\"http://www.php.net/manual/en/function.session-regenerate-id.php\"><kbd>session_regenerate_id</kbd></a> after the login prevents session fixation.</p>\n<p>Every page of Adminer contains the <code>robots: noindex</code> meta information to hide the page from search engines.</p>\n\n<h3>Login form</h3>\n<p>Adminer saves the database credentials to a session and checks them on every page. If the database cannot authorize the user then the login form is displayed directly on the page. This approach is better then redirecting the user to a login page for several reasons:</p>\n<ol>\n<li>It saves one request, the login form is sent directly to the target page.</li>\n<li>User can bookmark any page, which displays directly after the login. The redirection to a login form approach can achieve this too but it is much more complicated (the login form needs to get the returning URL, check it for validity and redirect back after the login because the <code class=\"jush-http\">Referer</code> header is not reliable).</li>\n<li>If the session expires before sending any application form then the posted data prepopulates the login form with the hidden fields.</li>\n</ol>\n<?php\nfunction hidden_fields($process) {\n	while (list($key, $val) = each($process)) {\n		if (is_array($val)) {\n			foreach ($val as $k => $v) {\n				$process[$key . \"[$k]\"] = $v;\n			}\n		} else {\n			echo \'<input type=\"hidden\" name=\"\' . htmlspecialchars($key) . \'\" value=\"\' . htmlspecialchars($val) . \'\" />\';\n		}\n	}\n}\nhidden_fields($_POST);\n?>\n<p>The sent files are also stored to hidden fields and there is a little layer serving the files from the <kbd>$_FILES</kbd> variable or from the posted hidden fields.</p>\n<p><ins datetime=\"2009-06-05T15:00:00+02:00\">It is possible to log in without filling the form by passing the <code>?username=</code> parameter. A user can use this feature in a secure environment where he authenticates by some other mean (e.g. on localhost without a remote access or with HTTP authentication). However, it is not possible to pass the password in a GET parameter which would be very insecure. Adminer uses the value of <code>mysql.default_password</code> configuration directive in this case. The demo application uses this approach to automatically log in the users.</ins></p>\n\n<h3>Form processing</h3>\n<p>Adminer sends all action forms by the POST method. If the action is successful then it redirects the browser to a page showing the result with a confirmation message. This message is stored to a session variable after performing the action and the beginning of each HTML page processes this session variable. There is a theoretical chance that other Adminer page opened simultaneously in a different browser tab can catch the message and display it instead of the original page. However, the period between setting the message and displaying it is very short so this probability is low. Besides, the consequence of this coincidence would be small – the user would get informed about the result, just in a different tab.</p>\n<p>An alternative to this approach would be to send a message ID or the message itself in the URL but it would clutter the browser history. In addition, the page refresh would maintain the message on the page, which is not desirable.</p>\n<p>If the form action produces an error then the page directly displays the error message with the form prefilled by the sent data. The code does not redirect the browser in this case, which allows a user to refresh the page with the sent data if the error was only temporary (for example if it could be fixed in other browser tab).</p>\n\n<h3>Database schema</h3>\n<p><img src=\"data/adminer-schema.png\" width=\"189\" height=\"152\" alt=\"Simple database schema\" class=\"right\" /> Adminer contains a simple interface for visualization of the database schema. It displays all tables with their columns. Colors differentiate the column types and arrows symbolize the table references. Tables can be moved around the screen with a mouse, which is achieved by JavaScript and CSS. The database schema page uses a cookie to save the table placement.</p>\n\n<h3>Export</h3>\n<p>Adminer supports SQL and CSV import and export. The most interesting variant of export is the one using the ALTER command. There is often a different database on the production and the development server. If you have made some changes in the database structure which should go to the production server during a release then dropping the old table and creating the new one is not an option because there are live data in this table. Adminer is able to generate a set of ALTER commands synchronizing the databases. It utilizes a stored procedure which explores the production database and alters it to conform to the development version. New tables are created as usual, old tables are dropped, missing fields and indexes are added.</p>\n\n<h3>Performance</h3>\n<p>Adminer always queries all data directly from the database with one exception. Getting the database list could take a very long time if there are many databases on the server even if the user has access only to couple of them. Because each page displays the database list then it is cached to a session variable. This variable is refreshed if a user creates or drops a database either from the user interface or by the SQL command.</p>\n<p>Closing a session by <a href=\"http://www.php.net/manual/en/function.session-write-close.php\"><kbd>session_write_close</kbd></a> is a performance optimization with a little impact. The default PHP session handler locks the file with a session data, which prevents the web server to serve multiple requests of the same user at the same time. This is very important on sites using frames where it is a common situation. Adminer does not use frames but a user can still open several browser tabs with Adminer at the same time. Adminer writes all the session data and closes the session as soon as possible to allow the maximum concurrency.</p>\n<p>Most users feel the smoothness of Adminer and one user even created the test suite, which measures the performance with comparison to phpMyAdmin:</p>\n<ul>\n<li>Performance of the same operations is 28% faster in average (2 × 23 tests).</li>\n<li>Start after login is 56% faster (2 × 4 tests).</li>\n<li>Finally, understanding of the source code was 100% faster.</li>\n</ul>\n\n<h3>Demonstration</h3>\n<p>The demo is one of the best ways to present the features of some product. <span class=\"callout\">I have created some sample database and made it accessible from the website through Adminer but users have started corrupting it quickly</span>. I was thinking about periodically restoring the database but it would not work because in the meantime the database would be still corrupted. On the other side, users working with the database during the reset would be surprised what is happening. Restricting the actions would cripple the demo, so it was not an option either. Other possibility was to emulate the database access in a session for each user but it would be a very complex task and it will not present all the features of Adminer.</p>\n<p>Finally, I have ended with creating a separate database for each user entering the demo. The script initializes this database by a sample data after login to the demo and destroys it after logout. A cron job drops the databases of users who did not log out. The demo user has powerful privileges to present the most features of Adminer so it was necessary to restrict the list of databases to disallow accessing databases of other users. Each MySQL user can also modify her password and users of the demo application quickly discovered it. Thus, the demo disables also this feature.</p>\n<p>I did not want to create a separate version of Adminer just for the demo so I have made all customizations by <a href=\"http://www.php.net/manual/en/ini.core.php#ini.auto-prepend-file\"><kbd>auto_prepend_file</kbd></a>. The prepended script restricts the database list, disallows changing of password, initializes the demo databases and drops them.</p>\n\n<h3>Tests and code coverage</h3>\n<p>Nearly all parts of the application have tests. I have created the tests in <a href=\"http://seleniumhq.org/projects/ide/\"><strong>Selenium IDE</strong></a>, which is a very convenient Firefox plug-in for creating complex web applications tests. The main advantage of Selenium is that it can test the whole application – from PHP on the server side to HTML on the client side, it can test even the JavaScript interactions. Adminer works well without JavaScript but some features are more comfortable with the JavaScript enabled – for example adding a field in a table does not require sending the page to the server.</p>\n<p>Selenium IDE can create the tests very easily. It is possible to just push the record button and work in a browser as usually. The recorded test can be modified afterwards so if you are checking some feature in the application then why not to record a test for it?</p>\n<p>I was curious how much code is tested and I was interested in which parts of Adminer need tests that are more thorough. It can be find out by the code coverage. I have used the <a href=\"http://www.xdebug.org/\"><strong>Xdebug</strong></a> extension, which has the ability to measure the code coverage. It is well suited for running unit tests because the code coverage is computed for the whole run of the tests. But the web application tests are different as they compose from several requests and the code coverage is computed for each request separately. The solution was quite easy – I have registered a shutdown function saving the code coverage to a session and joining it with the previous results.</p>\n<p><span class=\"callout\">The code coverage of Adminer is around 75 %</span>, which is satisfactory. For comparison, the <a href=\"http://gcov.php.net/\">code coverage of PHP source codes</a> is around 70 %. I was very glad that the tests exist when I refactored some parts of the application. I was quite sure that the refactoring did not break anything when all the tests have passed.</p>\n\n<h3>Version checker</h3>\n<p>There is a RSS feed with the project news and e-mail subscriptions for the new releases. Some users however just install the current version and forget about it. For this reason, the Adminer contains a version checker.</p>\n<p>The version checker must be as unobtrusive as possible because sometimes the Adminer is run in an environment without the Internet connection or the server could be unreachable. So checking the current version from PHP code was not an option because it is blocking. Furthermore, an administrator can disable the remote connections in PHP by <a href=\"http://www.php.net/manual/en/filesystem.configuration.php#ini.allow-url-fopen\"><kbd>allow_url_fopen</kbd></a> configuration directive. Thus, the version checker runs from JavaScript. A simple <code class=\"jush\">&lt;script src&gt;</code> is blocking too (a user can work with the page but the page is still in the loading state) which is the same for images too. So the script is loaded in an <code>onload</code> event which is non-blocking. The server part is quite simple:</p>\n<?php\nheader(\"Content-Type: text/javascript\");\nif (version_compare($_GET[\"version\"], $VERSION) < 0) {\n	echo \"document.getElementById(\'version\').innerHTML = \'$VERSION\';\\n\";\n}\necho \"document.cookie = \'adminer_version=$VERSION\';\\n\";\n?>\n<p>If the current version is old then the script modifies the document to display the new version number. The client side checks the sent cookie so the communication with the server is only occasional.</p>\n\n<h3>Design</h3>\n<p><img src=\"data/adminer-paranoiq.png\" width=\"447\" height=\"218\" alt=\"Design from Vlasta Neubauer\" class=\"right\" /> I am not an artist so the style sheet comes from freelance graphic designer Ondřej Válka. The design is simple and tidy as the whole Adminer so I like it. Other users however like more fancy designs so they created their own. The design can be easily integrated to the single file by own compilation but users like to test several designs so Adminer checks if the external file <code>adminer.css</code> exists. If it does exist then the script uses it instead of the integrated CSS.</p>\n\n<h3>Conclusion</h3>\n<p>This article is about the Adminer architecture, not about its features. However, it should be clear that Adminer supports all MySQL features from a simple table editing, through multiple-columns foreign keys, triggers, stored routines, export, user and processes management to the MySQL 5.1 events and table partitioning. There is of course a universal SQL command for non-standard tasks.</p>\n<p>Adminer is not just a compact version of phpMyAdmin, it tries to be a fully competitive alternative for the MySQL administration and the small footprint is only a pleasant bonus.</p>\n\n<h3><a name=\"author\"></a>About the author</h3>\n<p>Jakub Vrána is one of the authors of official PHP manual, and he uses MySQL for developing web applications in PHP. He also teaches the PHP and MySQL basics at Charles University and conducts commercial trainings. He is the founder of compact MySQL management tool Adminer. Jakub lives in Prague, Czech Republic.</p>\n',	'2009-12-28',	0),
(561,	1,	'Variable initialization in PHP',	'<p><em>This article was published as <a href=\"http://php-security.org/2010/05/17/mops-submission-06-variable-initialization-in-php/\">the Month of PHP Security Submission</a>.</em></p>\n\n<h3><a name=\"intro\"></a>Introduction</h3>\n<p>Consider the following code:</p>\n<?php\nif (authUser($_POST[\"login\"], $_POST[\"password\"])) {\n	$auth = true;\n}\nif ($auth) {\n	echo \"Secret\\n\";\n}\n?>\n<p>You can easily spot the vulnerability in it. The <code class=\"jush-php\">$auth</code> variable is not initialized in all cases so it can be spoofed from outside. PHP defines a way to handle uninitialized variables (unlike C language for example), they all have the <code class=\"jush-php\">null</code> value. The problem is that the variable can be initialized from other sources:</p>\n<ul>\n<li>Previous code</li>\n<li>Included file</li>\n<li>From outside if <code class=\"jush-phpini\">register_globals</code> is enabled</li>\n<li>By using of <code class=\"jush-php\">extract</code> on an untrusted variable</li>\n</ul>\n\n<h3><a name=\"defense\"></a>Defense</h3>\n<p>The defense against this vulnerability is simple – always initialize all variables:</p>\n<?php\n$auth = false;\nif (authUser($_POST[\"login\"], $_POST[\"password\"])) {\n	$auth = true;\n}\n?>\n<p>Now if <code class=\"jush-php\">$auth</code> contained anything (or nothing) then it is always reset to <code class=\"jush-php\">false</code>. It is a good idea to initialize variables unconditionally. The following code would work but it is not so robust:</p>\n<?php\nif (authUser($_POST[\"login\"], $_POST[\"password\"])) {\n	$auth = true;\n} else {\n	$auth = false;\n}\n?>\n<p>Now consider that someone would like to check also the IP address and do it in this way:</p>\n<?php\nif ($_SERVER[\"REMOTE_ADDR\"] == \"127.0.0.1\") {\n	if (authUser($_POST[\"login\"], $_POST[\"password\"])) {\n		$auth = true;\n	}\n} else {\n	$auth = false;\n}\n?>\n<p>The <code class=\"jush-php\">$auth</code> variable can be spoofed again.</p>\n<p>Note: The attacker must know the variable name to spoof – he can guess it, get it from some error message, brute-force it, but most commonly he gets it from the source code (open-source applications or a former employee).</p>\n\n<h3><a name=\"globals\"></a>Disabling register_globals</h3>\n<p>It is important to mention that disabling <code class=\"jush-phpini\">register_globals</code> is not a defense against this attack. It only closes the most common attack vector. It is of course a good idea to disable it but good applications that initialize all variables work independently of <code class=\"jush-phpini\">register_globals</code> value. Thus, it is not necessary to emulate disabling of <code class=\"jush-phpini\">register_globals</code> by some variation of the following code:</p>\n<?php\n// never use this kind of code\nif (ini_get(\"register_globals\")) {\n	foreach ($_REQUEST as $key => $val) {\n		unset($$key);\n	}\n}\n?>\n\n<h3><a name=\"notice\"></a>Spotting of uninitialized variables</h3>\n<p>PHP has a mechanism to spot uninitialized variables – it issues <code class=\"jush-php\">E_NOTICE</code> level error with most uses of uninitialized variables. There are however some problems with it:</p>\n<ol>\n<li>It does not warn about assigning to an uninitialized array.</li>\n<li>It warns about accessing non-existing index in properly initialized array.</li>\n<li>It is issued in runtime.</li>\n</ol>\n\n<h4><a name=\"array-assing\"></a>E_NOTICE does not warn about assigning to an uninitialized array</h4>\n<p>The first problem is most serious, consider the following code:</p>\n<?php\n$config[\"password\"] = \"pwd\";\nif (isset($_POST[\"password\"]) && $_POST[\"password\"] == $config[\"password\"]) {\n	echo \"Secret information.\\n\";\n}\n?>\n<p>It issues no notices if an attacker spoofs the <code class=\"jush-php\">$config</code> variable. If he passes a string then the code is interpreted like this:</p>\n<?php\n$config[0] = \"p\";\n// $config is a string so [] is used to access bytes in the string\n// \"password\" is converted to number 0 because string positions are always integers\n// only one byte can be written to [0] so \"pwd\" is interpreted as \"p\"\nif (isset($_POST[\"password\"]) && $_POST[\"password\"] == $config[0]) {\n	echo \"Secret information.\\n\";\n}\n?>\n<p>Now it is enough to guess the first character of password and send it along with spoofed <code class=\"jush-php\">$config</code>.</p>\n\n<h4><a name=\"array-access\"></a>E_NOTICE warns about accessing non-existing index in properly initialized array</h4>\n<p>The second problem is not security related but code-brevity related. Following code would issue a notice even if it works well and cannot be fooled:</p>\n<pre class=\"jush\">&lt;input name=\"search\" value=\"&lt;?php echo htmlspecialchars((string) $_GET[\"search\"]); ?&gt;\" /&gt;\n</pre>\n<p>With notices enabled, we must rewrite it to a longer code with the same functionality:</p>\n<pre class=\"jush\">&lt;input name=\"search\" value=\"&lt;?php\nif (isset($_GET[\"search\"])) {\n	echo htmlspecialchars((string) $_GET[\"search\"]);\n}\n?&gt;\" /&gt;\n</pre>\n<p>Another example of this thoroughness – following code is perfectly valid and does what we expect from it (count group members):</p>\n<?php\n$groups = array();\nforeach (getData() as $data) {\n	$groups[$data[\"id_group\"]]++;\n}\n?>\n<p>With notices enabled, it must be rewritten like this:</p>\n<?php\n$groups = array();\nforeach (getData() as $data) {\n	if (!isset($groups[$data[\"id_group\"]])) {\n		$groups[$data[\"id_group\"]] = 0;\n	} else {\n		$groups[$data[\"id_group\"]]++;\n	}\n}\n?>\n<p>I would not say that this code is more clear and less error-prone (there is already one error included).</p>\n\n<h4><a name=\"runtime\"></a>E_NOTICE is issued in runtime</h4>\n<p>The third problem of notices is security-related. If some usage of uninitialized variable is not spotted during the development then an attacker can use it. It is nice that you are informed about the usage of uninitialized variable from the error log but if it is used for an attack then it is too late. It is possible to make notices fatal by <code class=\"jush-php\">set_error_handler</code> but it is not worth it for most applications.</p>\n\n<h3><a name=\"initialized\"></a>Better spotting of uninitialized variables</h3>\n<p>I would not recommend disabling notices. However, it does not solve all problems and requires writing of more thorough code as you have seen. Luckily, there is a better way to spot uninitialized variables that solves all three problems of notices. It has a name <a href=\"http://code.google.com/p/php-initialized/\"><strong>php-initialized</strong></a>. It is a tool for analyzing PHP source code to spot uninitialized variables. It does not run the code and has some limitations but it can be used to check the code routinely for example after refactoring or before commit.</p>\n<p>The biggest limitation is that only the current block is considered as the variable scope. Thus, the following code would complain about uninitialized <code class=\"jush-php\">$auth</code>:</p>\n<?php\nif (authUser($_POST[\"login\"], $_POST[\"login\"])) {\n	$auth = true;\n} else {\n	$auth = false;\n}\nif ($auth) {\n	echo \"Secret\\n\";\n}\n// prints: Uninitialized variable $auth on line 7\n?>\n<p>It is partially a political decision explained in this article – it is less error-prone to initialize variables unconditionally. Apart this limitation, the supported <a href=\"http://code.google.com/p/php-initialized/wiki/Features\">features</a> covers nearly all parts of PHP.</p>\n<p>Note: Author of this article is the main developer of <strong>php-initialized</strong>.</p>\n\n<h3><a name=\"request\"></a>Do not use $_REQUEST</h3>\n<p>Variable spoofing does not involve only global variables. The <code class=\"jush-php\">$_REQUEST</code> variable can be filled by other source than a programmer assumes. It is <a href=\"http://www.suspekt.org/2008/10/01/php-53-and-delayed-cross-site-request-forgerieshijacking/\">thoroughly explained</a> by Steffan Esser. I would only append that as the contents of <code class=\"jush-php\">$_REQUEST</code> can be affected by <code class=\"jush-phpini\">request_order</code> since PHP 5.3.0 then an application could not rely on the contents of <code class=\"jush-php\">$_REQUEST</code> and would not run on some configurations.</p>\n\n<h3><a name=\"summary\"></a>Summary</h3>\n<p>Good PHP application should always initialize all variables before usage. It is a good idea to turn off <code class=\"jush-phpini\">register_globals</code> but a good application should not rely on it. PHP offers an <code class=\"jush-php\">E_NOTICE</code> error level that can spot some uninitialized variables but it is not 100% reliable. Tool <a href=\"http://code.google.com/p/php-initialized/\"><strong>php-initialized</strong></a> solves the deficiencies of it.</p>\n',	'2010-05-17',	0),
(562,	1,	'Context-aware HTML escaping',	'<p><em>This article was published as <a href=\"http://php-security.org/2010/05/05/mops-submission-02-context-aware-html-escaping/\">the Month of PHP Security Submission</a>.</em></p>\n\n<h3><a name=\"intro\"></a>Introduction</h3>\n<p>Cross-site scripting (XSS) is one of the most common vulnerabilities in web applications. Defense against this attack on HTML pages is however quite simple – it is enough to change characters <code class=\"jush\">&lt;</code> and <code class=\"jush\">&amp;</code> which have a special meaning in HTML text to the corresponding entities <code class=\"jush\">&amp;lt;</code> and <code class=\"jush\">&amp;amp;</code> just before outputting any untrusted data. If we want to output an untrusted data inside a quoted value of HTML attribute (like <code class=\"jush-tag\">title=\"\"</code>) we have to escape quotes to <code class=\"jush\">&amp;quot;</code>.</p>\n<p>PHP offers this escaping by a function <code class=\"jush-php\">htmlspecialchars</code> which encodes all three special characters (and <code class=\"jush\">&gt;</code> as a bonus). It is also possible to escape <code>\'</code> which can be used to quote HTML attribute value by passing a second parameter with value <code class=\"jush-php\">ENT_QUOTES</code>.</p>\n<p>It is important to note that this function can be safely used only to escape HTML text (data between tags without a special meaning like have <code class=\"jush\">&lt;script&gt;</code> or <code class=\"jush\">&lt;style&gt;</code>) or inside a quoted value of an attribute without a special meaning (like have <code class=\"jush-tag\">onmouseover=\"\"</code> or <code class=\"jush-tag\">style=\"\"</code>). Other contexts (like tag or attribute name, unquoted attribute value, or HTML comments) are unsafe even after usage of this function.</p>\n<p>It is also important to note that the page must explicitly set character set by <code class=\"jush-http\">Content-Type</code> header. It can be otherwise fooled to use character set where other characters have special meaning (like UTF-7). The character set can be also passed to <code class=\"jush-php\">htmlspecialchars</code> but it is not required with one-byte character sets or UTF-8.</p>\n<p>Note: <code class=\"jush-php\">htmlentities</code> function can be used with ancient encodings to encode some characters which do not exist in the character set. This function is not required with Unicode which covers all characters.</p>\n\n<h3><a name=\"automatic\"></a>Automatic escaping</h3>\n<p>If the defense against XSS is so simple (use <code class=\"jush-php\">htmlspecialchars</code> to any printed data) then why it is such a common vulnerability? The reason is that programmers often forget to use it. Sometimes they use it on usual XSS targets like discussions but they forget to use it in search or registration form. Another time they escape all forms but forget about URL parameters.</p>\n<p>The best way to not forget about escaping is to automate it. Most templating engines offer an automatic escaping. For example <strong>Smarty</strong> offers a <a href=\"http://www.smarty.net/manual/en/variable.default.modifiers.php\"><code class=\"jush-php\">$default_modifiers</code></a> variable which can add an <code>escape</code> filter escaping all printed data. These default modifiers can be disabled by <code>smarty:nodefaults</code> filter applied to any variable so it is still possible to output a trusted HTML code but it requires a longer code. It is an important observation – shorter code is more secure.</p>\n<p>I see this automatic escaping as one of the most important features of templating engines (other one is a separation of HTML and PHP code). Pure PHP templates do not offer this feature.</p>\n<p>Note: Other option is to generate an XML data from PHP script through <strong>DOM</strong> or other PHP extension and convert them to HTML by an <strong>XSL</strong> template. This approach is equivalent to automatic escaping because text content created by e.g. <a href=\"http://www.php.net/manual/en/domdocument.createtextnode.php\"><code>createTextNode</code></a> is serialized with special characters converted to entities. Creating such applications is however more difficult than using classic templates and requires more resources to generate the page.</p>\n\n<h3><a name=\"context\"></a>Context-aware escaping</h3>\n<p>There are still other contexts with different sets of special characters, most importantly the <code class=\"jush\">&lt;script&gt;</code> tag and JavaScript event handlers. They can be usually separated to an external file but sometimes not (for example initializing a user-specific JavaScript variable is better to do in the inline <code class=\"jush\">&lt;script&gt;</code> tag). Most importantly the decision of the data usage is up to the template author. He can decide to use the data escaped for HTML inside a JavaScript event. The important part of our application security is in hands of an HTML coder! This is often a guy with brilliant color sensitivity who however hardly understood loops.</p>\n<p>The solution of this problem lies in the context-aware escaping which improves the automatic escaping to recognize the context and choose escaping function appropriate for this context.</p>\n<p>The first templating engine with context-aware escaping is probably the Google\'s <a href=\"http://code.google.com/p/google-ctemplate/\"><strong>ctemplate</strong></a> which is available for C++. The only context-aware escaping template engine for PHP known to the author is <a href=\"http://doc.nettephp.com/en/template-filters\"><strong>Nette Latte</strong></a> which is a part of the <a href=\"http://nettephp.com/\"><strong>Nette Framework</strong></a> but can also be used independently.</p>\n<p>Note: The Nette Framework is created with emphasis on security which is visible not only in templates but also in all other parts. For example the defense against Cross-site request forgery in the framework forms is easy.</p>\n\n<h3><a name=\"latte\"></a>Nette Latte</h3>\n<p>Nette Latte templating engine automatically recognizes following contexts:</p>\n<ul>\n<li>HTML text</li>\n<li>HTML attribute value quoted in <code>\"</code> or <code>\'</code></li>\n<li>Tags <code class=\"jush\">&lt;script&gt;</code> and <code class=\"jush\">&lt;style&gt;</code> with CDATA section</li>\n<li>HTML attributes <code class=\"jush-tag\">style</code> and <code>on*</code> (like <code class=\"jush-tag\">onclick</code> or <code class=\"jush-tag\">onmousemove</code>)</li>\n<li>HTML comments</li>\n</ul>\n<p>This allows writing even complicated (but still realistic) code without any manual escaping:</p>\n<pre class=\"jush\">&lt;script type=\"text/javascript\"&gt;\nvar userId = {$userId};\n&lt;/script&gt;\n&lt;p style=\"color: {$color};\" title=\"{$title}\"&gt;\n&lt;a href=\"\" onclick=\"return !confirm({$message});\"&gt;{$desc}&lt;/a&gt;\n&lt;/p&gt;\n&lt;!-- Executed in: {$time} s --&gt;\n</pre>\n<p>If you try to escape this code by hand without any restrictions on the variable values then you will probably find it very difficult.</p>\n<p>Please note that variables used inside the JavaScript code are unquoted. Consider them as usual JS variables – PHP numbers are printed as JS numbers, PHP strings as JS strings, PHP associative arrays as JS object literals and so on.</p>\n<p>Automatic context-aware escaping can\'t be disabled by some magic filter (like in Smarty) but there is a separate syntax to print a raw variable value: <code>{!$var}</code>. Again – less code means more security, moreover exclamation mark points to something possibly dangerous.</p>\n<p>Note: There is no special context for URLs. The reason is that links are created with a separate tag <code>{link}</code>.</p>\n\n<h3><a name=\"summary\"></a>Summary</h3>\n<p>Escaping of HTML special characters is simple but it can be easily forgotten. Automatic escaping solves this problem but doesn\'t respect contexts with different special characters such as JavaScript. Context-aware escaping comes to the rescue. Nette Latte is a solid templating engine for PHP with this feature.</p>\n',	'2010-05-05',	0),
(563,	1,	'Save test',	'...',	'2010-11-12',	0);

DROP TABLE IF EXISTS `category`;
CREATE TABLE `category` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `name` varchar(100) COLLATE utf8_unicode_ci NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=3 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

INSERT INTO `category` (`id`, `name`) VALUES
(1,	'English'),
(2,	'Czech');

DROP TABLE IF EXISTS `rticle_ag`;
CREATE TABLE `rticle_ag` (
  `rticle_id` int(11) NOT NULL,
  `ag_id` int(11) NOT NULL,
  PRIMARY KEY (`rticle_id`,`ag_id`),
  KEY `rticle_ag_ag_id_idx` (`ag_id`),
  CONSTRAINT `rticle_ag_ibfk_2` FOREIGN KEY (`ag_id`) REFERENCES `tag` (`id`) ON DELETE CASCADE,
  CONSTRAINT `rticle_ag_ibfk_1` FOREIGN KEY (`rticle_id`) REFERENCES `article` (`id`) ON DELETE CASCADE
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

INSERT INTO `rticle_ag` (`rticle_id`, `ag_id`) VALUES
(446,	1),
(499,	1),
(174,	2),
(499,	2),
(561,	2),
(562,	2),
(563,	2),
(174,	3),
(561,	3),
(562,	3),
(446,	6),
(499,	6),
(563,	6);

DROP TABLE IF EXISTS `tag`;
CREATE TABLE `tag` (
  `id` int(11) NOT NULL AUTO_INCREMENT,
  `name` varchar(100) COLLATE utf8_unicode_ci NOT NULL,
  PRIMARY KEY (`id`)
) ENGINE=InnoDB AUTO_INCREMENT=7 DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci;

INSERT INTO `tag` (`id`, `name`) VALUES
(1,	'MySQL'),
(2,	'PHP'),
(3,	'Security'),
(4,	'HTML'),
(5,	'i18n'),
(6,	'Performance');

-- 2010-11-12 00:42:25
